/*
 * Copyright (c) Citrix Systems, Inc
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <stdlib.h>
#include <stdint.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/wait.h>

#include <backend.h>
#include <debug.h>
#include <oxtdb.h>
#include <xapidb.h>

#include "option.h"

/* Path to the file used for resuming. */
static char *arg_resume;
/* Path to the file used for saving. */
static char *arg_save;

static bool
oxtdb_parse_arg(const char *name, const char *val)
{
    if (!strcmp(name, "resume"))
        arg_resume = strdup(val);
    else if (!strcmp(name, "save"))
        arg_save = strdup(val);
    else if (!strcmp(name, "uuid"))
        oxtdb_arg_uuid = strdup(val);
    else
        return false;

    return true;
}

static bool
oxtdb_check_args(void)
{
    if (!oxtdb_arg_uuid) {
        fprintf(stderr, "Backend arg 'uuid' is required\n");
        return false;
    }
    if (!opt_resume && arg_resume) {
        fprintf(stderr, "Backend arg 'resume' is invalid when not resuming\n");
        return false;
    }

    return true;
}

static bool
oxtdb_save(void)
{
    /* not supported or used on OpenXT */
    return false;
}

static bool
oxtdb_resume(void)
{
    /* not supported or used on OpenXT */
    return false;
}

const struct backend oxtdb = {
    .parse_arg = oxtdb_parse_arg,
    .check_args = oxtdb_check_args,
    .init = oxtdb_init,
    .save = oxtdb_save,
    .resume = oxtdb_resume,
    .set_variable = oxtdb_set_variable,
    .sb_notify = oxtdb_sb_notify,
};
